/* ASCII text file transfer. Uses XON / OFF flow control protocol.
** Transfer ASCII files only.
*/

#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>

#include "pcl4c.h"
#include "modem.h"

extern int GetChar();
extern int PutChar();

char lastXchar;     /* last XON or XOFF sent */
int echo = FALSE;   /* local echo flag */

int TxAscii(Port,Filename,Buffer,Length)
int Port;            /* COM port [0..3] */
char Filename[];     /* filename buffer */
char Buffer[];       /* data buffer */
int Length;          /* size of Buffer */
{int i;
 int Code;           /* return code */
 int Handle;         /* file Handle */
 char LastChar;      /* last character sent */
 int TxChars = 0;    /* # characters transmitted */
 int Count;          /* # bytes read from disk */
 char Temp[81];      /* temporary buffer */
 /* begin */
 lastXchar =  XON;
 /* Filename is not empty */
 Handle = open(Filename,O_RDONLY|O_BINARY,S_IREAD);
 if(Handle<0)
     {strcpy(Temp,"Cannot open ");
      strcat(Temp,Filename);
      DisplayLine(Temp,NULL,0);
      return(FALSE);
     }
 DisplayLine("ASCII: Starting send",NULL,0);
 while(SioKeyPress()) SioKeyRead();
 /* clear comm port */
 SioRxFlush(Port);
 /* send ascii file ( stop at ^Z ) */
 while(1)
     {/* read next buffer from disk */
      Count = read(Handle,Buffer,Length);
      if(Count==0) break;
      if(Count<0)
          {SayError(Port,"Error on disk read");
           return(FALSE);
          }
      /* send one byte at a time */
      for(i=0;i<Count;i++)
          {/* send byte */
           LastChar = Buffer[i];
           if(echo) printf("%c",LastChar);
           PutChar(Port,LastChar);
           TxChars++;
           /* ^Z marks the end of a text file */
           if(LastChar==CTLZ) break;
           /* check for incoming ^Z */
           if(lastXchar==XON)
                {/* check for incoming XON / XOFF */
                 Code = GetChar(Port,0);
                 if(Code>0)
                    {/* is byte a XOFF ? */
                     if((char)Code==XOFF)
                         {/* wait for XON */
                          DisplayLine("XOFF received",NULL,0);
                          lastXchar = XOFF;
                          /* wait for XON */
                          while(1)
                              {/* user want to quit ? */
                               if(SioKeyPress())
                                   {DisplayLine("Canceled by USER",NULL,0);
                                    return(FALSE);
                                   }
                               Code = GetChar(Port,ONESECOND);
                               if((char)Code==XON)
                                   {DisplayLine("XON  received",NULL,0);
                                    lastXchar = XON;
                                    break;
                                   }
                              } /* end -- while */
                         } /* end -- if(XOFF) */
                    } /* end -- if(Code) */
                } /* end -- if(XON) */

          } /* end -- for(i) */
     } /* end -- while */
 close(Handle);
 if(LastChar!=CTLZ) PutChar(Port,CTLZ);
 DisplayLine("Ascii Transfer Complete",NULL,0);
 return(TRUE);
} /* end -- TxAcsii */

int RxAscii(Port,Filename,Buffer,Length,RxQueSize)
int Port;            /* COM port [0..3] */
char Filename[];     /* filename buffer */
char Buffer[];       /* data buffer */
int Length;          /* length of data buffer */
int RxQueSize;       /* size of PCL receive buffer */
{int i;
 int Handle;         /* file Handle */
 int Code;           /* return code */
 int lo;             /* receive queue low water mark */
 int hi;             /* receive queue high water mark */
 int Index;          /* buffer index */
 int QueSize;        /* current PCL receive queue size */
 int RxChars = 0;    /* # received chars */
 int Count;          /* # characters written to disk */
 char Temp[81];      /* temporary buffer */
 /* begin */
 lastXchar = XON;
 lo = RxQueSize / 4;
 hi = 3 * lo;
 DisplayLine("ASCII: Starting receive ",NULL,0);
 while(SioKeyPress()) SioKeyRead();
 /* clear comm port */
 SioRxFlush(Port);
 /* open file passed in Filename[] for write */
 Handle = open(Filename,O_CREAT|O_TRUNC|O_WRONLY|O_BINARY,S_IWRITE);
 if(Handle<0)
     {strcpy(Temp,"Cannot open ");
      strcat(Temp,Filename);
      DisplayLine(Temp,NULL,0);
      return(FALSE);
     }
 /* receive text */
 Index = 0;
 while(1)
     {/* user want to quit ? */
      if(SioKeyPress())
          {DisplayLine("Canceled by USER",NULL,0);
           return(FALSE);
          }
      /* check on PCL receive queue size */
      QueSize = SioRxQue(Port);
      if((QueSize>hi)&&(lastXchar==XON))
          {PutChar(Port,XOFF);
           lastXchar = XOFF;
           DisplayLine("sending XOFF",NULL,0);
          }
      if((QueSize<lo)&&(lastXchar==XOFF))
          {PutChar(Port,XON);
           lastXchar = XON;
           DisplayLine("sending XON ",NULL,0);
          }
      /* get next byte */
      Code = GetChar(Port,ONESECOND);
      if(Code==-1) continue;
      /* ignore 1st character if it is a 0 */
      if((RxChars==0)&&((char)Code=='\0')) continue;
      /* received a character */
      Buffer[Index++] = (char)Code;
      RxChars++;
      /* ^Z marks the end of a text file */
      if((char)Code==CTLZ)
          {/* add 2nd ^Z */
           Buffer[Index++] = CTLZ;
           RxChars++;
           break;
          }
      if(echo) printf("%c",(char)Code);
      if(Index==Length)
          {/* send XOFF to sender */
           PutChar(Port,XOFF);
           lastXchar = XOFF;
           DisplayLine("sending XOFF",NULL,0);
           /* write disk file */
           Count = write(Handle,Buffer,Index);
           if(Count<0)
               {SayError("Disk read error",NULL);
                SioDelay(18);
                return(FALSE);
               }
           /* send XON to sender */
           PutChar(Port,XON);
           lastXchar = XON;
           DisplayLine("sending XON ",NULL,0);
           Index = 0;
          } /* end -- if */
     } /* end -- while */
 /* write any remaining data */
 if(Index>0)
     {Count = write(Handle,Buffer,Index);
      if(Count<0)
          {SayError("Disk read error",NULL);
           SioDelay(18);
           return(FALSE);
          }
     } /* end -- if */
 close(Handle);
 DisplayLine("ASCII Transfer Complete",NULL,0);
 return(TRUE);
} /* RxAscii */
