/*
**                    ---  term.c ---
**
**  EXAMPLE CODE: Terminal emulator. Can transfer files using Ascii.
**  Registered users can also use XMODEM, and YMODEM protocols.
**
**  NOTE: shareware users must link with AMODEM, CRC, DOS, and PCL.LIB
**  Registered users link with AMODEM, XYMODEM, XYPACKET, CRC, DOS, and PCL.LIB
*/

#define XYMODEM 0

#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>

#include "pcl4c.h"
#include "modem.h"

/*** Global variables ***/

int Port;               /* current COM port [0..3] */
char Filename[15];      /* file name buffer */
char Buffer[1024];      /* block buffer */
char far RxBuf[1024];   /* !!! Note that RxBuf[] is defined as FAR !!! */
char *BaudRate[10] = {"300","600","1200","2400","4800","9600",
                      "19200","38400","57600","115200"};
int AsciiFlag = TRUE;
int CRCflag   = TRUE;
int OneKflag  = FALSE;
int BatchFlag = FALSE;

/*** main program ***/

main(argc,argv)
int argc;
char *argv[];
{int DataFlag = FALSE;
 char c;
 int i, k;
 int n, rc;
 int Delta;         /* delta port status */
 int Status;        /* port status */
 int BaudCode;      /* baud rate code ( index into BaudRate[] ) */
 char temp[81];
 /* right number of parameters ? */
 if(argc!=3)
     {printf("Usage: 'TERM port baud' -- example 'TERM 1 9600'\n");
      exit(1);
     }
 /* get port number from command line */
 Port = atoi(argv[1]) - 1;
 if((Port<0) || (Port>3))
     {printf("Port must be 1 to 4\n");
      exit(1);
     }
 /* get baud rate from command line */
 BaudCode = BaudMatch(argv[2]);
 if(BaudCode<0)
     {printf("Cannot recognize baud rate = %s\n",argv[2]);
      exit(1);
     }
 /* setup receive buffer */
 rc = SioRxBuf(Port,RxBuf,Size1024);
 if(rc<0)
     {SioError(rc);
      exit(1);
     }
 /* set parms & reset (initialize) COM port */
 SioParms(Port,NoParity,OneStopBit,WordLength8);
 MyStart(Port,BaudCode);
 /* init CRCR table */
 InitCRC();
 /* set DTR and RTS */
 SioDTR(Port,'S');
 SioRTS(Port,'S');
#if 0
 printf("Waiting for DSR.");
 while( !SioModem(Port,DSR) )
     {
      if(SioKeyPress()||SioBrkKey()) MyExit(0,"Aborted by user");
      putchar('.');
      SioDelay(ONESECOND);
     }
 putchar('\n');
#endif
 /* initialize screen */
 Scroll(0,0,24,79,0,NORMAL);
 /* display status message */
 sprintf(temp," COM%d %s 'ESC for menu' ",1+Port,BaudRate[BaudCode]);
 for(i=0;i<strlen(temp);i++)
     {/* write character */
      Position(24,50+i);
      AttrWrite(temp[i],INVERSE);
     }
 /* home cursor & clear CCL receive buffer */
 Position(0,0);
 SioRxFlush(Port);
 /* enter terminal loop */
 SioRxFlush(Port);
 while(1)
     {/* Control-BREAK ? */
      if(SioBrkKey()) MyExit(0,"User pressed Ctrl-BREAK");
      /* was key pressed ? */
      if(SioKeyPress())
          {/* read key press */
           i = SioKeyRead();
           if((char)i==ESC)
               {/* process user's request */
                ProcessESC();
                continue;
               }
           else PutChar(Port,i);
          }
      /* was break detected ? */
      if( SioBrkSig(Port,'D') ) DisplayLine("BREAK detected ",NULL,0);
      /* any incoming over serial port ? */
      i = GetChar(Port,0);
      if(i>-1)
          {/* good character */
           if(DataFlag&(i<0x20))
               {MyCrtWrite('^');
                MyCrtWrite('@'+i);
               }
           else MyCrtWrite(i&0x007f);
          }
      /* any change in DCD or DSR ? */
      Delta = SioModem(Port,DeltaDCD|DeltaDSR);
      if(Delta)
          {/* display new status */
           Status = SioModem(Port,DCD|DSR);
           if(!Delta&DeltaDCD) MyExit(0,"Dropped DSD");
           if(!Delta&DeltaDSR) MyExit(0,"Dropped DSR");
          }
     } /* end -- key pressed */
}

/*** write to screen except for bottom line ***/

int MyCrtWrite(ch)
char ch;
{/* write character */
 SioCrtWrite(ch);
 /* scroll all but bottom line */
 if(GetRow()==24)
    {Scroll(0,0,23,79,1,NORMAL);
     Position(23,0);
    }
}

/*** make multiple attempts to reset port ***/

int MyStart(Port,BaudCode)
int Port;
int BaudCode;
{int i, rc;
 /* try up to 5 times to reset COM port */
 for(i=0;i<5;i++)
     {printf("Resetting COM%d at %s baud\n",Port+1,BaudRate[BaudCode]);
      if( (rc = SioReset(Port,BaudCode))==0) return;
      if(rc<0) MyExit(rc,"Error resetting port");
      SioDone(Port);
      /* display errors */
      if(rc&OverrunError) puts("Overrun Error");
      if(rc&ParityError)  puts("Parity Error");
      if(rc&FramingError) puts("Framing Error");
      if(rc&BreakDetect)  puts("Break Detect");
     }
 exit(1);
}

/*** find baud rate string in table ***/

int BaudMatch(ptr)
char *ptr;
{int i;
 /* find baud rate in table */
 for(i=0;i<10;i++) if(strcmp(BaudRate[i],ptr)==0) return(i);
 return(-1);
}

/*** user pressed Escape */

int ProcessESC()
{int i;
 int rc;
 int c1, c2;
 char Answer[2]; /* array for 1 char answer */
 int row, col;
 /* user pressed <ESC> */
 Answer[0] = '?';
 Answer[1] = '\0';
#if XYMODEM
 DisplayLine("Q)uit P)rotocol S)end R)eceive: ",Answer,1);
#else
 DisplayLine("Q)uit S)end R)eceive: ",Answer,1);
#endif
 if(strlen(Answer)) switch(toupper(Answer[0]))
     {
#if XYMODEM
      case 'P':
          DisplayLine("A)SCII X)MODEM Y)MODEM: ",Answer,1);
          if(strlen(Answer)) switch( toupper(Answer[0]) )
               {
                case 'A':
                    AsciiFlag = TRUE;
                    DisplayLine("Protocol = ASCII",NULL,1);
                    break;
                case 'X':
                    AsciiFlag = FALSE;
                    OneKflag = FALSE;
                    BatchFlag = FALSE;
                    CRCflag = TRUE;
                    DisplayLine("Protocol = XMODEM",NULL,1);
                    break;
                case 'Y':
                    AsciiFlag = FALSE;
                    OneKflag = TRUE;
                    BatchFlag = TRUE;
                    CRCflag = TRUE;
                    DisplayLine("Protocol = YMODEM",NULL,1);
                    break;
                default:
                    DisplayLine("Must answer A, X, or Y",NULL,1);
                    break;
               }
          break;
#endif
      case 'Q':
          MyExit(0,"User pressed ESC");
          break;
      case 'R':
          if(AsciiFlag)
               {/* ASCII receive */
                DisplayLine("Enter filename:",Filename,15);
                if(strlen(Filename)==0) break;
                RxAscii(Port,Filename,Buffer,1024,1024);
                break;
               }
#if XYMODEM
          /* XMODEM / YMODEM receive */
          if(BatchFlag)
               {do
                    {/* receive files till get empty filename */
                     RxyModem(Port,Filename,Buffer,CRCflag,BatchFlag);
                     if(SioKeyPress()) break;
                    } while(Filename[0]!='\0');
               }
          else /* not Batch */
               {DisplayLine("Enter filename:",Filename,15);
                if(strlen(Filename)==0) break;
                RxyModem(Port,Filename,Buffer,CRCflag,BatchFlag);
               }
#endif
          break;
      case 'S':
          DisplayLine("Enter filename:",Filename,15);
          if(strlen(Filename)==0) break;
          if(AsciiFlag)
               {/* ASCII send */
                TxAscii(Port,Filename,Buffer,1024);
               }
#if XYMODEM
          else
               {/* XMODEM / YMODEM send */
                TxyModem(Port,Filename,Buffer,OneKflag,BatchFlag);
                if(BatchFlag)
                    {/* send empty filename */
                     Filename[0] = '\0';
                     SioDelay(5);
                     TxyModem(Port,Filename,Buffer,OneKflag,BatchFlag);
                     }
               }
#endif
          break;
     default:
#if XYMODEM
          DisplayLine("Must answer Q, P, S, or R",NULL,0);
#else
          DisplayLine("Must answer Q, S, or R",NULL,0);
#endif
          break;
    } /* end switch */
}

/*** Display status line ***/

#define RIGHTMOST 40
int DisplayLine(MsgPtr,UserPtr,UserLength)
char *MsgPtr;
char *UserPtr;
int UserLength;
{static int right = RIGHTMOST;
 int row;
 int col;
 int i;
 char c;
 int MsgLength;
 row = GetRow();
 col = GetCol();
 Position(24,0);
 /* display message */
 MsgLength = strlen(MsgPtr);
 for(i=0;i<MsgLength;i++)
     {AttrWrite(*MsgPtr++,INVERSE);
      Position(24,i+1);
     }
 /* blank rest of menu bar */
 for(i=MsgLength;i<right;i++)
     {AttrWrite(' ',INVERSE);
      Position(24,i+1);
     }
 right = MsgLength;
 /* want response from user ? */
 if(UserPtr!=NULL)
     {right = RIGHTMOST;
      /* input text from user */
      i = 0;
      while(1)
          {Position(24,MsgLength+i);
           c = SioKeyRead();
           if((c=='\r')||(c==ESC)||(MsgLength+i==RIGHTMOST))
               {UserPtr[i] = '\0';
                break;
               }
           if((c==BS)&&(i>0))
               {i--;
                Position(24,MsgLength+i);
                AttrWrite(' ',INVERSE);
               }
           else
               {/* save character & display on status line */
                UserPtr[i++] = c;
                AttrWrite(c,INVERSE);
                if(i==UserLength)
                    {/* user string done */
                     UserPtr[i] = '\0';
                     break;
                    }
               }
          } /* end -- while */
     } /* end -- if */
 Position(row,col);
}
int PutChar(Port,ch)
int Port;
char ch;
{int rc;
 rc = SioPutc(Port,ch);
 if(rc<0)
     {printf("COM%d Error: ",1+Port);
      SioError(rc);
      exit(1);
     }
 return(rc);
 }

int GetChar(Port,timeout)
int Port;
int timeout;
{int rc;
 rc = SioGetc(Port,timeout);
 if(rc<-1)
     {printf("COM%d Error: ",1+Port);
      SioError(rc);
      exit(1);
     }
 return(rc);
}

int SayError(Port,ptr)
int Port;
char *ptr;
{char temp[81];
 sprintf(temp,"ERROR! COM%d : %s",1+Port,ptr);
 DisplayLine(temp,NULL,0);
 printf("\n*** %s",temp);
 /* cancel remote */
 PutChar(Port,CAN);
 PutChar(Port,CAN);
 PutChar(Port,CAN);
}

int MyExit(code,msg)
int code;
char *msg;
{int rc;
 if(code<0) SioError(code);
 /* Assert UART break & sign off */
 SioBrkSig(Port,'A');
 printf("\nTERMINATING: %s\n",msg);
 SioDelay(ONESECOND);
 if( (rc = SioDone(Port)) < 0 ) SioError(rc);
 exit(0);
}